.container {
  background-color: white;
  width: 80%;
  max-width: 1900px;
}

.file-input {
  height: 60px;
  line-height: 60px;
  border-width: 1px;
  border-style: dashed;
  border-radius: 5px;
  text-align: center;
  margin: 10px;
}

.col {
  margin-right: auto;
  margin-left: unset;
}